
import 'dart:convert';
import 'package:flutter/material.dart';
import 'package:airpay_emar_package/model/Transaction.dart';
import 'package:airpay_emar_package/model/UserRequest.dart';
import 'package:flutter_spinkit/flutter_spinkit.dart';
import 'package:http/http.dart' as http;
import 'package:encrypt/encrypt.dart' as encrypt;
import 'package:airpay_emar_package/model/QrDecryptResponse.dart';
import 'package:crypto/crypto.dart';


typedef void CallBack(CARDDETAILS response);
typedef void QRDecryptClosure(bool val, QrDecryptResponse response);


class QRDecrypt extends StatefulWidget {
  final QRDecryptClosure? qrDecryptClosure;
  final UserRequest user;

  QRDecrypt({Key? key, required this.user,this.qrDecryptClosure}) : super(key: key);
  @override
  _QRDecryptState createState() => new _QRDecryptState();
}

class _QRDecryptState extends State<QRDecrypt> {
  bool isShowProgress = true;
  var privateKey;
  var email;
  var mobileNo;
  var tokenData;
  var merchantId;
  var secret_key ='b0B1c#dsaf45ADSF5645adf0215';

  @override
  void initState() {
    super.initState();
    var user = widget.user;
    makeApiCall(user.query);
  }

  showLoader(value) {
    setState(() {
      isShowProgress = value;
    });
  }


  // Function to make API call
  Future<void> makeApiCall(qrString) async {

    String stagingDataURL = "https://emar.airpay.ninja/node/api/qrDecrypt";
    String productionDataURL = "https://api.airpay.ae/node/api/qrDecrypt";
    String stagingAirpayKey = "AL0013AL1688469524";
    String stagingPartnerKey = "AF0064AF1688469603";
    String productionAirpayKey = "AL0015AL1693490184";
    String productionPartnerKey = "AF0038AF1693490253";

    var isGateWay = (widget.user.isStaging != null && widget.user.isStaging == true)
        ? stagingDataURL
        : productionDataURL;
    String url = isGateWay;

    var airpayKey = (widget.user.isStaging != null && widget.user.isStaging == true)
        ? stagingAirpayKey
        : productionAirpayKey;
    String airpay_Key = airpayKey;

    var partnerKey = (widget.user.isStaging != null && widget.user.isStaging == true)
        ? stagingPartnerKey
        : productionPartnerKey;
    String partner_Key = partnerKey;



    var checksum_header = sha256.convert(utf8.encode('${qrString.toString()}${secret_key.toString()}'));

    // Define your request headers
    Map<String, String> headers = {
      'AIRPAYKEY': airpay_Key,
      'PARTNERKEY': partner_Key,
      'CHECKSUM': checksum_header.toString(),
      'Content-Type': 'application/x-www-form-urlencoded'

    };

    // Define your request body
    Map<String, dynamic> requestBody = {
      'query': qrString,
    };


    try {
      // Send POST request to the API
      print("QR DECRYPT url -->"+url);

      final response = await http.post(
        Uri.parse(url),
        headers: headers,
        body: requestBody,

      );


      if (response.statusCode == 200) {
        // Request successful, handle the response here

        // Parse the response body as JSON
        Map<String, dynamic> parsedResponseStatus = jsonDecode(response.body);

        // Check if the 'status' key is present in the response
        if (parsedResponseStatus.containsKey('status')) {
          // Access the 'status' and 'message' properties in the parsed response
          int status = parsedResponseStatus['status'];
          String message = parsedResponseStatus['message'];

          widget.qrDecryptClosure!(true, QrDecryptResponse(status: status,message: message,data: null));
          return;
          // The rest of your code...
        } else  if (parsedResponseStatus.containsKey('query')) {
          // Parse the response body as JSON
          Map<String, dynamic> parsedResponse = jsonDecode(response.body);
          // Access the 'query' property in the parsed response
          String query = parsedResponse['query'];
          String decryptResponse = decryptStr(query);

          widget.qrDecryptClosure!(true, qrDecryptResponseFromJson(decryptResponse));
          return;
        }

      } else {
        // Request failed, handle the error here
        print('API call failed');
        userCancel("Service Unavailable");
      }
    } catch (error) {
      print('Error EXception : $error');
      userCancel("Service Unavailable");
    }


  }

  //Decryption Code for QR Decrypt API
  String decryptStr(String fullResponse){
    fullResponse.split("|");
    // String ivFromResponse = fullResponse.substring(0, 16);
    //String encryptedResponse = fullResponse.substring(16);
    String ivFromResponse = fullResponse.split("|")[0];
    String encryptedResponse = fullResponse.split("|")[1];

    final key = encrypt.Key.fromUtf8("5PbpCqeioHQ3MWj6dJVeluXD");
    final iv = encrypt.IV.fromUtf8(ivFromResponse);
    //final encrypter = encrypt.Encrypter(encrypt.AES(key,mode:encrypt.AESMode.cbc));
    final encrypter = encrypt.Encrypter(encrypt.AES(key, mode: encrypt.AESMode.cbc)); //, padding: 'PKCS7'


    final encrypted = encrypt.Encrypted.fromBase16(encryptedResponse);
    final decrypted = encrypter.decrypt(encrypted, iv: iv);

    return decrypted;
  }


  userCancel(String withMessage) {

    widget.qrDecryptClosure!(true,QrDecryptResponse(status: 500,message: withMessage,data: null));

  }


  @override
  Widget build(BuildContext context) {
    return SafeArea(
        child: Scaffold(
            appBar: AppBar(
              centerTitle: true,
              title: Image.asset(
                'assets/airpayLogo.png',
                height: 40,
                color: Colors.white,
                width: 200,
              ),
              leading: IconButton(
                icon: Icon(Icons.close),
                onPressed: () {
                  Navigator.pop(context);
                },
              ),
              backgroundColor: Colors.blue[900],
              actions: <Widget>[],
            ),
            body: isShowProgress ?
            Container(
                child: Center(
                    child: SpinKitCircle(
                      color: Colors.blue[900],
                      size: 50.0,
                    ))) : SizedBox()
        )
    );
  }
}
