
import 'dart:convert';
import 'dart:math';
import 'package:airpay_emar_package/model/CardDetailResponse.dart';
import 'package:airpay_emar_package/model/CardTokenResponse.dart';
import 'package:crypto/crypto.dart';
import 'package:flutter/material.dart';
import 'package:airpay_emar_package/model/Transaction.dart';
import 'package:airpay_emar_package/model/UserRequest.dart';
import 'package:dio/dio.dart';
import 'package:flutter_spinkit/flutter_spinkit.dart';
import 'package:airpay_emar_package/model/MerchantDetailResponse.dart';

import 'package:http/http.dart' as http;
import 'package:encrypt/encrypt.dart' as encrypt;

typedef void CallBack(CARDDETAILS response);
//typedef void Closure(bool val, Transaction response);
typedef void CardDetailClosure(bool val, CardDetailResponse response);

class CardDetails extends StatefulWidget {
  final CardDetailClosure? cardDetailClosure;
  final UserRequest user;

  CardDetails({Key? key, required this.user,this.cardDetailClosure}) : super(key: key);
  @override
  _CardDetailsState createState() => new _CardDetailsState();
}

class _CardDetailsState extends State<CardDetails> {
  bool isShowProgress = true;
  var privatekey;
  var email;
  var mobileNo;
  var tokenData;
  // var merchantId;
  String airpayMerchantId = "";
  String airpayApikey = "";
  String airpayUsername = "";
  String airpayPassword = "";
  String transaction_session_id = "";
  String successUrl = "";
  String failedUrl = "";
  var secret_key ='b0B1c#dsaf45ADSF5645adf0215';

  @override
  void initState() {
    super.initState();
    // var user = widget.user;
    email = widget.user.buyerEmail;
    mobileNo = widget.user.buyerPhone;

    loadData();

  }

  showLoader(value) {
    setState(() {
      isShowProgress = value;
    });
  }

  Future<void> loadData() async {
    Map<String, dynamic> jsonParams = {
      "merchant_id": widget.user.mercid.toString(),
      "order_id": widget.user.orderid.toString(),
      "amount": widget.user.amount.toString(),
      "discount": widget.user.discount.toString(),
      "customvar": widget.user.customvar.toString(),
      "isQR": widget.user.isQR.toString(),

    };
    String jsonString = json.encode(jsonParams);
    var merchant_response = await merchantDetailsApi(jsonString);
    if(merchant_response!= null) {
      if (merchant_response.status
          .toString()
          .isNotEmpty && merchant_response.status.toString() == "200") {
        airpayApikey = merchant_response.data!.airpayApikey.toString();
        airpayMerchantId = merchant_response.data!.airpayMerchantId.toString();
        airpayUsername = merchant_response.data!.airpayUsername.toString();
        airpayPassword = merchant_response.data!.airpayPassword.toString();
        transaction_session_id =
            merchant_response.data!.transactionSessionId.toString();
        successUrl = merchant_response.data!.successUrl.toString();
        failedUrl = merchant_response.data!.failedUrl.toString();

        var temp = utf8.encode(
            '${airpayApikey}@${airpayUsername}:|:${airpayPassword}');
        privatekey = sha256.convert(temp);

        fetchTokenDetails(privatekey);
      }else{
        /*widget.cardDetailClosure!(true,"");
        showLoader(false);*/
      }
    }


  }

  fetchTokenDetails(privateKey) async {
    try {
      showLoader(true);
      FormData formData = new FormData.fromMap({
        "action": "getConsumerToken",
        "type": "consumer",
        "respType": "json",
        "mercid": airpayMerchantId,
        "privatekey": privatekey,
        "email": email,
        "mobile": mobileNo
      });

      String stagingDataURL = 'https://uae-ca.airpay.ninja/api.php';
      String productionDataURL = 'https://ca.airpay.ae/api.php';
      var isGateWay = (widget.user.isStaging != null &&
          widget.user.isStaging == true)
          ? stagingDataURL
          : productionDataURL;

      String url = isGateWay;

      Dio dio = new Dio();
      dio.options.headers['Accept'] =
      'text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8';
      var response = await dio.post(url, data: formData);
      var cardTokenResponse = cardTokenResponseFromJson(response.toString());

      fetchCardListDetails(cardTokenResponse.result![0].token.toString());
    }catch (error) {
      print('Error Exception In TOKEN : $error');

    }
  }

  fetchCardListDetails(String token) async {
    try {
      FormData formData = new FormData.fromMap({
        "action": "listCardAPI",
        "type": "card",
        "respType": "json",
        "mercid": airpayMerchantId,
        "privatekey": privatekey,
        "token": token
      });

      String stagingDataURL = 'https://uae-ca.airpay.ninja/api.php';
      String productionDataURL = 'https://ca.airpay.ae/api.php';
      var isGateWay = (widget.user.isStaging != null &&
          widget.user.isStaging == true)
          ? stagingDataURL
          : productionDataURL;

      String url = isGateWay;

      Dio dio = new Dio();
      dio.options.headers['Accept'] =
      'text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8';
      var response = await dio.post(url, data: formData);
      var respData = response.data;

      String cardResponse = response.toString();
      var cardDetailResponse = cardDetailResponseFromJson(cardResponse);
      widget.cardDetailClosure!(true, cardDetailResponse);
      showLoader(false);
// Navigator.pop(context);
    }catch (error) {
      print('Error Exception In CARD DETAIL : $error');

    }
  }

  userCancel(String withMessage) {
    var trans = Transaction();
    trans.statusCode = '500';
    trans.message = withMessage;
  }

  // Function to make API call
  Future<MerchantDetailResponse?> merchantDetailsApi(json_request) async {
    try{

      String stagingDataURL = 'https://emar.airpay.ninja/node/api/getMerchantDetails';
      String productionDataURL = 'https://api.airpay.ae/node/api/getMerchantDetails';
      String stagingAirpayKey = "AL0013AL1688469524";
      String stagingPartnerKey = "AF0064AF1688469603";
      String productionAirpayKey = "AL0015AL1693490184";
      String productionPartnerKey = "AF0038AF1693490253";

      var isGateWay = (widget.user.isStaging != null &&
          widget.user.isStaging == true)
          ? stagingDataURL
          : productionDataURL;

      String url = isGateWay;

      var airpayKey = (widget.user.isStaging != null &&
          widget.user.isStaging == true)
          ? stagingAirpayKey
          : productionAirpayKey;
      String airpay_Key = airpayKey;

      var partnerKey = (widget.user.isStaging != null &&
          widget.user.isStaging == true)
          ? stagingPartnerKey
          : productionPartnerKey;
      String partner_Key = partnerKey;


      var merchant_request = encryptStr(json_request);
      var checksum_header = sha256.convert(
          utf8.encode('${merchant_request.toString()}${secret_key.toString()}'));
      // Define your request headers
      Map<String, String> headers = {
        'AIRPAYKEY': airpay_Key,
        'PARTNERKEY': partner_Key,
        'CHECKSUM': checksum_header.toString(),
        'Content-Type': 'application/x-www-form-urlencoded'
      };

      // Define your request body
      Map<String, dynamic> requestBody = {
        'query': merchant_request,
      };


      // Send POST request to the API
      final response = await http.post(
        Uri.parse(url),
        headers: headers,
        body: requestBody,

      );

      if (response.statusCode == 200) {
        // Request successful, handle the response here

        // Parse the response body as JSON
        Map<String, dynamic> parsedResponse_status = jsonDecode(response.body);

        // Check if the 'status' key is present in the response
        if (parsedResponse_status.containsKey('status')) {
          // Access the 'status' and 'message' properties in the parsed response
          int status = parsedResponse_status['status'];
          String message = parsedResponse_status['message'];

          return MerchantDetailResponse(
              status: status, message: message, data: null);
          // The rest of your code...
        } else if (parsedResponse_status.containsKey('query')) {
          // Parse the response body as JSON
          Map<String, dynamic> parsedResponse = jsonDecode(response.body);
          // Access the 'query' property in the parsed response
          String query = parsedResponse['query'];
          String decrypt_response = decryptStr(query);
          var merchant_details_response = merchantDetailResponseFromJson(
              decrypt_response);

          return merchant_details_response;
        }
        else {
          return null;
        }
      } else {
        // Request failed, handle the error here
        print('API call failed');
        return null;
      }
    } catch (error) {
      print('Error EXception : $error');
      userCancel("Service Unavailable");
    }
  }

  //Decryption code - MS
  String decryptStr(String fullResponse){
    fullResponse.split("|");

    String ivFromResponse = fullResponse.split("|")[0];
    String encryptedResponse = fullResponse.split("|")[1];

    final key = encrypt.Key.fromUtf8("5PbpCqeioHQ3MWj6dJVeluXD");
    final iv = encrypt.IV.fromUtf8(ivFromResponse);
    //final encrypter = encrypt.Encrypter(encrypt.AES(key,mode:encrypt.AESMode.cbc));
    final encrypter = encrypt.Encrypter(encrypt.AES(key, mode: encrypt.AESMode.cbc)); //, padding: 'PKCS7'


    final encrypted = encrypt.Encrypted.fromBase16(encryptedResponse);
    final decrypted = encrypter.decrypt(encrypted, iv: iv);

    return decrypted;
  }

  //Encryption code - MS
  String encryptStr(String plainText){

    var generatedIV = generate16DigitNumber();
    final key = encrypt.Key.fromUtf8("5PbpCqeioHQ3MWj6dJVeluXD");
    // final iv = encrypt.IV.fromSecureRandom(16);
    final iv = encrypt.IV.fromUtf8(generatedIV);
    final encrypter = encrypt.Encrypter(encrypt.AES(key,mode:encrypt.AESMode.cbc));

    final encrypted = encrypter.encrypt(plainText, iv: iv);
    final decrypted = encrypter.decrypt(encrypted, iv: iv);

    return generatedIV+"|"+encrypted.base16;
  }

  String generate16DigitNumber() {
    Random random = Random();
    String number = '';
    for (int i = 0; i < 16; i++) {
      number += random.nextInt(10).toString();
    }
    return number;
  }


  @override
  Widget build(BuildContext context) {
    return SafeArea(
        child: Scaffold(
            appBar: AppBar(
              centerTitle: true,
              title: Image.asset(
                'assets/airpayLogo.png',
                height: 40,
                color: Colors.white,
                width: 200,
              ),
              leading: IconButton(
                icon: Icon(Icons.close),
                onPressed: () {
                  Navigator.pop(context);
                },
              ),
              backgroundColor: Colors.blue[900],
              actions: <Widget>[],
            ),
            body: isShowProgress ?
            Container(
                child: Center(
                    child: SpinKitCircle(
                      color: Colors.blue[900],
                      size: 50.0,
                    ))) : SizedBox()
        )
    );
  }
}
